from pyglet.gl import *
import data
from math import *
import random
from particles import *

class GameObject:
    MAP = "MAP"
    HERO = "HERO"
    ENEMY = "ENEMY"
    ENEMY1 = "ENEMY1"
    PICKUP = "PICKUP"
    OBJECT = "OBJECT"
    def __init__(self, r = 15):
        self.pos = (100, 100)
        self.r = r 
        self.vel = (10, 10)
        self.bounds = (800, 600)
        self.type = GameObject.OBJECT

    def update(self, dt):
        pos = (self.pos[0] + self.vel[0], self.pos[1] + self.vel[1])
        #set the velocity for bounce
        pvel = self.vel
        if pos[0]-self.r < 0 or pos[0]+self.r > self.bounds[0]:
            self.vel = (-self.vel[0], self.vel[1])
        if pos[1]-self.r < 0 or pos[1]+self.r > self.bounds[1]:
            self.vel = (self.vel[0], -self.vel[1])
        if pvel == self.vel:
            self.pos = pos

    def render(self):
        glBegin(GL_POLYGON)
        glTexCoord2f(0, 0)
        glVertex2f(self.pos[0]-self.r, self.pos[1]-self.r)
        glTexCoord2f(0, 1)
        glVertex2f(self.pos[0]-self.r, self.pos[1]+self.r)
        glTexCoord2f(1, 1)
        glVertex2f(self.pos[0]+self.r, self.pos[1]+self.r)
        glTexCoord2f(1, 0)
        glVertex2f(self.pos[0]+self.r, self.pos[1]-self.r)
        glEnd()

    def collision(self, obj):
        x = self.pos[0] - obj.pos[0]
        y = self.pos[1] - obj.pos[1]
        d = sqrt(x*x + y*y)
        if d < self.r + obj.r:
            return True
        return False

class OutEnemy(GameObject):
    def __init__(self, r):
        GameObject.__init__(self)
        self.type = GameObject.ENEMY1
        self.wr = r #radius of the world outside
        self.angle = random.randint(0, 360) 
        self.adt = random.choice([-1,1])
        self.pos = (self.wr, 0)
        self.update(0)

    def render(self):
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, data.e1_image.id)

        glPushMatrix()
        glTranslatef(self.pos[0], self.pos[1], 0)
        glRotatef(self.angle, 0, 0, 1)
        glTranslatef(-self.pos[0], -self.pos[1], 0)
        GameObject.render(self)
        glPopMatrix()

        glDisable(GL_TEXTURE_2D)

    def update(self, dt):
        self.angle += self.adt;
        self.angle %= 360
        rads = self.angle * 2 * 3.14 / 360
        self.pos = (self.wr * cos(rads), self.wr*sin(rads))


class InEnemy(GameObject):
    def __init__(self):
        GameObject.__init__(self, 15)
        self.type = GameObject.ENEMY
        self.r_angle = 0

    def update(self, dt):
        self.r_angle += 60*dt
        self.r_angle %= 360

    def render(self):
        glPushMatrix()
        glTranslatef(self.pos[0], self.pos[1], 0)
        glRotatef(self.r_angle, 0, 0, 1)
        glTranslatef(-self.pos[0], -self.pos[1], 0)
        glColor3f(1, 1, 1)
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, data.e2_image.id)
        GameObject.render(self)
        glDisable(GL_TEXTURE_2D)
        glPopMatrix()

class PickUp(GameObject):
    def __init__(self):
        GameObject.__init__(self, 10)
        self.type = GameObject.PICKUP
        self.vel = (0, 0)
        self.r_angle = 0

    def update(self, dt):
        self.r_angle += 60 * dt;
        self.r_angle %= 360

    def render(self):
        glPushMatrix()
        glTranslatef(self.pos[0], self.pos[1], 0)
        glRotatef(self.r_angle, 0, 0, 1)
        glTranslatef(-self.pos[0], -self.pos[1], 0)
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, data.pickup_image.id)
        GameObject.render(self)
        glDisable(GL_TEXTURE_2D)
        glPopMatrix()

class Hero(GameObject):
    def __init__(self, r):
        GameObject.__init__(self)
        self.bound_r = r
        self.type = GameObject.HERO
        self.smoke = []
        self.add_smoke = 0

    def render(self):
        glEnable(GL_TEXTURE_2D)
        glBindTexture(GL_TEXTURE_2D, data.hero_image.id)
        GameObject.render(self)
        for smoke in self.smoke:
            smoke.render()
        glColor4f(1, 1, 1, 1)
        glDisable(GL_TEXTURE_2D)

    def update(self, dt):
        pos = (self.pos[0] + self.vel[0], self.pos[1]+ self.vel[1])
        if pos[0]*pos[0] + pos[1]*pos[1] > self.bound_r * self.bound_r:
            self.vel = (0, 0)
            self.add_smoke = 0
        else:
            self.pos = pos
            if not self.vel == (0, 0):
                self.add_smoke+= 1
            if self.add_smoke > 2:
                ppos = (self.pos[0] - 2*self.vel[0], self.pos[1]-2*self.vel[1])
                self.smoke.append(Particle(ppos, (0, 0), data.smoke_image.id))
                self.add_smoke = 0
        for smoke in self.smoke:
            smoke.update(dt)
            if smoke.size > 2 * Particle.MaxSize:
                self.smoke.remove(smoke)
